@@ -28,31 +28,42 @@ from operator import attrgetter
 import os
 import stat
 import time
+import subprocess
+import codecs
 
 from google import auth
+from prometheus_client import start_http_server
 from turbinia import config
 from turbinia.config import logger
 from turbinia.config import DATETIME_FORMAT
 from turbinia import task_manager
 from turbinia import TurbiniaException
 from turbinia.lib import text_formatter as fmt
+from turbinia.lib import docker_manager
+from turbinia.jobs import manager as job_manager
 from turbinia.workers import Priority
 from turbinia.workers.artifact import FileArtifactExtractionTask
 from turbinia.workers.analysis.wordpress import WordpressAccessLogAnalysisTask
 from turbinia.workers.analysis.jenkins import JenkinsAnalysisTask
-from turbinia.workers.binary_extractor import BinaryExtractorTask
+from turbinia.workers.analysis.jupyter import JupyterAnalysisTask
 from turbinia.workers.finalize_request import FinalizeRequestTask
+from turbinia.workers.docker import DockerContainersEnumerationTask
 from turbinia.workers.grep import GrepTask
 from turbinia.workers.hadoop import HadoopAnalysisTask
 from turbinia.workers.hindsight import HindsightTask
+from turbinia.workers.partitions import PartitionEnumerationTask
 from turbinia.workers.plaso import PlasoTask
 from turbinia.workers.psort import PsortTask
+from turbinia.workers.redis import RedisAnalysisTask
 from turbinia.workers.sshd import SSHDAnalysisTask
 from turbinia.workers.strings import StringsAsciiTask
 from turbinia.workers.strings import StringsUnicodeTask
 from turbinia.workers.tomcat import TomcatAnalysisTask
 from turbinia.workers.volatility import VolatilityTask
 from turbinia.workers.worker_stat import StatTask
+from turbinia.workers.binary_extractor import BinaryExtractorTask
+from turbinia.workers.bulk_extractor import BulkExtractorTask
+from turbinia.workers.photorec import PhotorecTask
 
 MAX_RETRIES = 10
 RETRY_SLEEP = 60
@@ -64,11 +75,14 @@ TASK_MAP = {
     'wordpressaccessloganalysistask': WordpressAccessLogAnalysisTask,
     'finalizerequesttask': FinalizeRequestTask,
     'jenkinsanalysistask': JenkinsAnalysisTask,
+    'JupyterAnalysisTask': JupyterAnalysisTask,
     'greptask': GrepTask,
     'hadoopanalysistask': HadoopAnalysisTask,
     'hindsighttask': HindsightTask,
+    'partitionenumerationtask': PartitionEnumerationTask,
     'plasotask': PlasoTask,
     'psorttask': PsortTask,
+    'redisanalysistask': RedisAnalysisTask,
     'sshdanalysistask': SSHDAnalysisTask,
     'stringsasciitask': StringsAsciiTask,
     'stringsunicodetask': StringsUnicodeTask,
@@ -76,6 +90,9 @@ TASK_MAP = {
     'volatilitytask': VolatilityTask,
     'stattask': StatTask,
     'binaryextractor': BinaryExtractorTask,
+    'bulkextractortask': BulkExtractorTask,
+    'dockertask': DockerContainersEnumerationTask,
+    'photorectask': PhotorecTask
 }
 
 config.LoadConfig()
@@ -86,7 +103,7 @@ if config.TASK_MANAGER.lower() == 'psq':
   from google.cloud import datastore
   from google.cloud import pubsub
 
-  from turbinia.lib.google_cloud import GoogleCloudFunction
+  from libcloudforensics.providers.gcp.internal import function as gcp_function
 elif config.TASK_MANAGER.lower() == 'celery':
   from turbinia.state_manager import RedisStateManager
 
@@ -94,6 +111,103 @@ log = logging.getLogger('turbinia')
 logger.setup()
 
 
+def get_turbinia_client(run_local=False):
+  """Return Turbinia client based on config.
+
+  Returns:
+    Initialized BaseTurbiniaClient or TurbiniaCeleryClient object.
+  """
+  config.LoadConfig()
+  # pylint: disable=no-else-return
+  if config.TASK_MANAGER.lower() == 'psq':
+    return BaseTurbiniaClient(run_local=run_local)
+  elif config.TASK_MANAGER.lower() == 'celery':
+    return TurbiniaCeleryClient(run_local=run_local)
+  else:
+    msg = 'Task Manager type "{0:s}" not implemented'.format(
+        config.TASK_MANAGER)
+    raise TurbiniaException(msg)
+
+
+def check_docker_dependencies(dependencies):
+  """Checks docker dependencies.
+
+  Args:
+    dependencies(dict): dictionary of dependencies to check for.
+
+  Raises:
+    TurbiniaException: If dependency is not met.
+  """
+  #TODO(wyassine): may run into issues down the line when a docker image
+  # does not have bash or which installed. (no linux fs layer).
+  log.info('Performing docker dependency check.')
+  job_names = list(job_manager.JobsManager.GetJobNames())
+  images = docker_manager.DockerManager().list_images(return_filter='short_id')
+
+  # Iterate through list of jobs
+  for job, values in dependencies.items():
+    if job not in job_names:
+      log.warning(
+          'The job {0:s} was not found or has been disabled. Skipping '
+          'dependency check...'.format(job))
+      continue
+    docker_image = values.get('docker_image')
+    # short id only pulls the first 10 characters of image id.
+    if docker_image and len(docker_image) > 10:
+      docker_image = docker_image[0:10]
+
+    if docker_image in images:
+      for program in values['programs']:
+        cmd = 'type {0:s}'.format(program)
+        stdout, stderr, ret = docker_manager.ContainerManager(
+            values['docker_image']).execute_container(cmd, shell=True)
+        if ret != 0:
+          raise TurbiniaException(
+              'Job dependency {0:s} not found for job {1:s}. Please install '
+              'the dependency for the container or disable the job.'.format(
+                  program, job))
+      job_manager.JobsManager.RegisterDockerImage(job, values['docker_image'])
+    elif docker_image:
+      raise TurbiniaException(
+          'Docker image {0:s} was not found for the job {1:s}. Please '
+          'update the config with the correct image id'.format(
+              values['docker_image'], job))
+
+
+def check_system_dependencies(dependencies):
+  """Checks system dependencies.
+
+  Args:
+    dependencies(dict): dictionary of dependencies to check for.
+
+  Raises:
+    TurbiniaException: If dependency is not met.
+  """
+  log.info('Performing system dependency check.')
+  job_names = list(job_manager.JobsManager.GetJobNames())
+
+  # Iterate through list of jobs
+  for job, values in dependencies.items():
+    if job not in job_names:
+      log.warning(
+          'The job {0:s} was not found or has been disabled. Skipping '
+          'dependency check...'.format(job))
+      continue
+    elif not values.get('docker_image'):
+      for program in values['programs']:
+        cmd = 'type {0:s}'.format(program)
+        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
+        output, _ = proc.communicate()
+        log.debug(
+            'Dependency resolved: {0:s}'.format(output.strip().decode('utf8')))
+        ret = proc.returncode
+        if ret != 0:
+          raise TurbiniaException(
+              'Job dependency {0:s} not found in $PATH for the job {1:s}. '
+              'Please install the dependency or disable the job.'.format(
+                  program, job))
+
+
 def check_directory(directory):
   """Checks directory to make sure it exists and is writable.
 
@@ -123,7 +237,7 @@ def check_directory(directory):
           'Can not add write permissions to {0:s}'.format(directory))
 
 
-class TurbiniaStats(object):
+class TurbiniaStats:
   """Statistics for Turbinia task execution.
 
   Attributes:
@@ -195,7 +309,7 @@ class TurbiniaStats(object):
         self.description, self.count, self.min, self.mean, self.max)
 
 
-class TurbiniaClient(object):
+class BaseTurbiniaClient:
   """Client class for Turbinia.
 
   Attributes:
@@ -289,7 +403,7 @@ class TurbiniaClient(object):
 
   def get_task_data(
       self, instance, project, region, days=0, task_id=None, request_id=None,
-      user=None, function_name='gettasks'):
+      user=None, function_name='gettasks', output_json=False):
     """Gets task data from Google Cloud Functions.
 
     Args:
@@ -301,12 +415,13 @@ class TurbiniaClient(object):
       task_id (string): The Id of the task.
       request_id (string): The Id of the request we want tasks for.
       user (string): The user of the request we want tasks for.
-      function_name (string): The GCF function we want to call
+      function_name (string): The GCF function we want to call.
+      output_json (bool): Whether to return JSON output.
 
     Returns:
-      List of Task dict objects.
+      (List|JSON string) of Task dict objects
     """
-    cloud_function = GoogleCloudFunction(project_id=project, region=region)
+    cloud_function = gcp_function.GoogleCloudFunction(project)
     func_args = {'instance': instance, 'kind': 'TurbiniaTask'}
 
     if days:
@@ -328,7 +443,8 @@ class TurbiniaClient(object):
     credential_error_count = 0
     while response is None and retry_count < MAX_RETRIES:
       try:
-        response = cloud_function.ExecuteFunction(function_name, func_args)
+        response = cloud_function.ExecuteFunction(
+            function_name, region, func_args)
       except auth.exceptions.RefreshError as exception:
         if credential_error_count == 0:
           log.info(
@@ -369,8 +485,17 @@ class TurbiniaClient(object):
           'Could not deserialize result [{0!s}] from GCF: [{1!s}]'.format(
               response.get('result'), e))
 
-    # Convert run_time/last_update back into datetime objects
     task_data = results[0]
+    if output_json:
+      try:
+        json_data = json.dumps(task_data)
+      except (TypeError, ValueError) as e:
+        raise TurbiniaException(
+            'Could not re-serialize result [{0!s}] from GCF: [{1!s}]'.format(
+                str(task_data), e))
+      return json_data
+
+    # Convert run_time/last_update back into datetime objects
     for task in task_data:
       if task.get('run_time'):
         task['run_time'] = timedelta(seconds=task['run_time'])
@@ -412,6 +537,27 @@ class TurbiniaClient(object):
       report.append('')
     return report
 
+  def format_worker_task(self, task):
+    """Formats a single task for Worker view.
+
+    Args:
+      task (dict): The task to format data for
+    Returns:
+      list: Formatted task data
+    """
+    report = []
+    report.append(
+        fmt.bullet('{0:s} - {1:s}'.format(task['task_id'], task['task_name'])))
+    report.append(
+        fmt.bullet(
+            'Last Update: {0:s}'.format(
+                task['last_update'].strftime(DATETIME_FORMAT)), level=2))
+    report.append(fmt.bullet('Status: {0:s}'.format(task['status']), level=2))
+    report.append(
+        fmt.bullet('Run Time: {0:s}'.format(str(task['run_time'])), level=2))
+    report.append('')
+    return report
+
   def format_task(self, task, show_files=False):
     """Formats a single task in short form.
 
@@ -600,10 +746,198 @@ class TurbiniaClient(object):
     report.append('')
     return '\n'.join(report)
 
+  def format_worker_status(
+      self, instance, project, region, days=0, all_fields=False):
+    """Formats the recent history for Turbinia Workers.
+
+    Args:
+      instance (string): The Turbinia instance name (by default the same as the
+          INSTANCE_ID in the config).
+      project (string): The name of the project.
+      region (string): The name of the zone to execute in.
+      days (int): The number of days we want history for.
+      all_fields (bool): Include historical Task information for the worker.
+    Returns:
+      String of Request status
+    """
+    # Set number of days to retrieve data
+    num_days = 7
+    if days != 0:
+      num_days = days
+    task_results = self.get_task_data(instance, project, region, days=num_days)
+    if not task_results:
+      return ''
+
+    # Sort task_results by last updated timestamp.
+    task_results = sorted(
+        task_results, key=itemgetter('last_update'), reverse=True)
+
+    # Create dictionary of worker_node: {{task_id, task_update,
+    # task_name, task_status}}
+    workers_dict = {}
+    unassigned_dict = {}
+    scheduled_counter = 0
+    for result in task_results:
+      worker_node = result.get('worker_name')
+      status = result.get('status')
+      status = status if status else 'No task status'
+      if worker_node and worker_node not in workers_dict:
+        workers_dict[worker_node] = []
+      elif not worker_node:
+        # Track scheduled/unassigned Tasks for reporting.
+        scheduled_counter += 1
+        worker_node = 'Unassigned'
+        if worker_node not in unassigned_dict:
+          unassigned_dict[worker_node] = []
+      if worker_node:
+        task_dict = {}
+        task_dict['task_id'] = result.get('id')
+        task_dict['last_update'] = result.get('last_update')
+        task_dict['task_name'] = result.get('name')
+        task_dict['status'] = status
+        # Check status for anything that is running.
+        if 'running' in status:
+          run_time = (datetime.now() -
+                      result.get('last_update')).total_seconds()
+          run_time = timedelta(seconds=run_time)
+          task_dict['run_time'] = run_time
+        else:
+          run_time = result.get('run_time')
+          task_dict['run_time'] = run_time if run_time else 'No run time.'
+        # Update to correct dictionary
+        if worker_node == 'Unassigned':
+          unassigned_dict[worker_node].append(task_dict)
+        else:
+          workers_dict[worker_node].append(task_dict)
+
+    # Generate report header
+    report = []
+    report.append(
+        fmt.heading1(
+            'Turbinia report for Worker activity within {0:d} days'.format(
+                num_days)))
+    report.append(
+        fmt.bullet('{0:d} Worker(s) found.'.format(len(workers_dict.keys()))))
+    report.append(
+        fmt.bullet(
+            '{0:d} Task(s) unassigned or scheduled and pending Worker assignment.'
+            .format(scheduled_counter)))
+    for worker_node, tasks in workers_dict.items():
+      report.append('')
+      report.append(fmt.heading2('Worker Node: {0:s}'.format(worker_node)))
+      # Append the statuses chronologically
+      run_status, queued_status, other_status = [], [], []
+      for task in tasks:
+        if 'running' in task['status']:
+          run_status.extend(self.format_worker_task(task))
+        elif 'queued' in task['status']:
+          queued_status.extend(self.format_worker_task(task))
+        else:
+          other_status.extend(self.format_worker_task(task))
+      # Add each of the status lists back to report list
+      not_found = [fmt.bullet('No Tasks found.')]
+      report.append(fmt.heading3('Running Tasks'))
+      report.extend(run_status if run_status else not_found)
+      report.append('')
+      report.append(fmt.heading3('Queued Tasks'))
+      report.extend(queued_status if queued_status else not_found)
+      # Add Finished Tasks
+      if all_fields:
+        report.append('')
+        report.append(fmt.heading3('Finished Tasks'))
+        report.extend(other_status if other_status else not_found)
+
+    # Add unassigned worker tasks
+    unassigned_status = []
+    for tasks in unassigned_dict.values():
+      for task in tasks:
+        unassigned_status.extend(self.format_worker_task(task))
+    # Now add to main report
+    if all_fields:
+      report.append('')
+      report.append(fmt.heading2('Unassigned Worker Tasks'))
+      report.extend(unassigned_status if unassigned_status else not_found)
+
+    return '\n'.join(report)
+
+  def format_request_status(
+      self, instance, project, region, days=0, all_fields=False):
+    """Formats the recent history for Turbinia Requests.
+
+    Args:
+      instance (string): The Turbinia instance name (by default the same as the
+          INSTANCE_ID in the config).
+      project (string): The name of the project.
+      region (string): The name of the zone to execute in.
+      days (int): The number of days we want history for.
+      all_fields (bool): Include all fields for the Request, which includes,
+          saved file paths.
+    Returns:
+      String of Request status
+    """
+    # Set number of days to retrieve data
+    num_days = 7
+    if days != 0:
+      num_days = days
+    task_results = self.get_task_data(instance, project, region, days=num_days)
+    if not task_results:
+      return ''
+
+    # Sort task_results by last updated timestamp.
+    task_results = sorted(
+        task_results, key=itemgetter('last_update'), reverse=True)
+
+    # Create dictionary of request_id: {saved_paths, last_update, requester,
+    # task_id}
+    request_dict = {}
+    for result in task_results:
+      request_id = result.get('request_id')
+      saved_paths = result.get('saved_paths')
+      if request_id not in request_dict:
+        saved_paths = set(saved_paths) if saved_paths else set()
+        request_dict[request_id] = {}
+        request_dict[request_id]['saved_paths'] = saved_paths
+        request_dict[request_id]['last_update'] = result.get('last_update')
+        request_dict[request_id]['requester'] = result.get('requester')
+        request_dict[request_id]['task_id'] = set([result.get('id')])
+      else:
+        if saved_paths:
+          request_dict[request_id]['saved_paths'].update(saved_paths)
+        request_dict[request_id]['task_id'].update([result.get('id')])
+
+    # Generate report header
+    report = []
+    report.append(
+        fmt.heading1(
+            'Turbinia report for Requests made within {0:d} days'.format(
+                num_days)))
+    report.append(
+        fmt.bullet(
+            '{0:d} requests were made within this timeframe.'.format(
+                len(request_dict.keys()))))
+    # Print report data for Requests
+    for request_id, values in request_dict.items():
+      report.append('')
+      report.append(fmt.heading2('Request ID: {0:s}'.format(request_id)))
+      report.append(
+          fmt.bullet(
+              'Last Update: {0:s}'.format(
+                  values['last_update'].strftime(DATETIME_FORMAT))))
+      report.append(fmt.bullet('Requester: {0:s}'.format(values['requester'])))
+      report.append(
+          fmt.bullet('Task Count: {0:d}'.format(len(values['task_id']))))
+      if all_fields:
+        report.append(fmt.bullet('Associated Evidence:'))
+        # Append all saved paths in request
+        for path in sorted(values['saved_paths']):
+          report.append(fmt.bullet(fmt.code(path), level=2))
+        report.append('')
+    return '\n'.join(report)
+
   def format_task_status(
       self, instance, project, region, days=0, task_id=None, request_id=None,
       user=None, all_fields=False, full_report=False,
-      priority_filter=Priority.HIGH):
+      priority_filter=Priority.HIGH, output_json=False):
     """Formats the recent history for Turbinia Tasks.
 
     Args:
@@ -621,14 +955,22 @@ class TurbiniaClient(object):
           summary.
       priority_filter (int): Output only a summary for Tasks with a value
           greater than the priority_filter.
+      output_json (bool): Whether to return JSON output.
 
     Returns:
-      String of task status
+      String of task status in JSON or human readable format.
     """
+    if user and days == 0:
+      days = 1000
     task_results = self.get_task_data(
-        instance, project, region, days, task_id, request_id, user)
+        instance, project, region, days, task_id, request_id, user,
+        output_json=output_json)
     if not task_results:
       return ''
+
+    if output_json:
+      return task_results
+
     # Sort all tasks by the report_priority so that tasks with a higher
     # priority are listed first in the report.
     for result in task_results:
@@ -697,7 +1039,7 @@ class TurbiniaClient(object):
     if not request.evidence:
       raise TurbiniaException('TurbiniaRequest does not contain evidence.')
     log.info('Running Task {0:s} locally'.format(task_name))
-    result = task.run_wrapper(request.evidence[0])
+    result = task.run_wrapper(request.evidence[0].serialize())
     return result
 
   def send_request(self, request):
@@ -725,7 +1067,7 @@ class TurbiniaClient(object):
 
     Returns: String of closed Task IDs.
     """
-    cloud_function = GoogleCloudFunction(project_id=project, region=region)
+    cloud_function = gcp_function.GoogleCloudFunction(project)
     func_args = {
         'instance': instance,
         'kind': 'TurbiniaTask',
@@ -734,11 +1076,11 @@ class TurbiniaClient(object):
         'user': user,
         'requester': requester
     }
-    response = cloud_function.ExecuteFunction('closetasks', func_args)
+    response = cloud_function.ExecuteFunction('closetasks', region, func_args)
     return 'Closed Task IDs: %s' % response.get('result')
 
 
-class TurbiniaCeleryClient(TurbiniaClient):
+class TurbiniaCeleryClient(BaseTurbiniaClient):
   """Client class for Turbinia (Celery).
 
   Overriding some things specific to Celery operation.
@@ -747,8 +1089,8 @@ class TurbiniaCeleryClient(TurbiniaClient):
     redis (RedisStateManager): Redis datastore object
   """
 
-  def __init__(self, *_, **__):
-    super(TurbiniaCeleryClient, self).__init__()
+  def __init__(self, *args, **kwargs):
+    super(TurbiniaCeleryClient, self).__init__(*args, **kwargs)
     self.redis = RedisStateManager()
 
   def send_request(self, request):
@@ -762,7 +1104,7 @@ class TurbiniaCeleryClient(TurbiniaClient):
   # pylint: disable=arguments-differ
   def get_task_data(
       self, instance, _, __, days=0, task_id=None, request_id=None,
-      function_name=None):
+      function_name=None, output_json=False):
     """Gets task data from Redis.
 
     We keep the same function signature, but ignore arguments passed for GCP.
@@ -780,26 +1122,33 @@ class TurbiniaCeleryClient(TurbiniaClient):
     return self.redis.get_task_data(instance, days, task_id, request_id)
 
 
-class TurbiniaServer(object):
+class TurbiniaServer:
   """Turbinia Server class.
 
   Attributes:
     task_manager (TaskManager): An object to manage turbinia tasks.
   """
 
-  def __init__(self, jobs_blacklist=None, jobs_whitelist=None):
+  def __init__(self, jobs_denylist=None, jobs_allowlist=None):
     """Initializes Turbinia Server.
 
     Args:
-      jobs_blacklist (Optional[list[str]]): Jobs we will exclude from running
-      jobs_whitelist (Optional[list[str]]): The only Jobs we will include to run
+      jobs_denylist (Optional[list[str]]): Jobs we will exclude from running
+      jobs_allowlist (Optional[list[str]]): The only Jobs we will include to run
     """
     config.LoadConfig()
     self.task_manager = task_manager.get_task_manager()
-    self.task_manager.setup(jobs_blacklist, jobs_whitelist)
+    self.task_manager.setup(jobs_denylist, jobs_allowlist)
 
   def start(self):
     """Start Turbinia Server."""
+    if config.PROMETHEUS_ENABLED:
+      if config.PROMETHEUS_PORT and config.PROMETHEUS_ADDR:
+        log.info('Starting Prometheus endpoint.')
+        start_http_server(
+            port=config.PROMETHEUS_PORT, addr=config.PROMETHEUS_ADDR)
+      else:
+        log.info('Prometheus enabled but port or address not set!')
     log.info('Running Turbinia Server.')
     self.task_manager.run()
 
@@ -808,19 +1157,47 @@ class TurbiniaServer(object):
     self.task_manager.add_evidence(evidence_)
 
 
-class TurbiniaCeleryWorker(TurbiniaClient):
+class TurbiniaCeleryWorker(BaseTurbiniaClient):
   """Turbinia Celery Worker class.
 
   Attributes:
     worker (celery.app): Celery worker app
   """
 
-  def __init__(self, *_, **__):
-    """Initialization for Celery worker."""
+  def __init__(self, jobs_denylist=None, jobs_allowlist=None):
+    """Initialization for celery worker.
+
+    Args:
+      jobs_denylist (Optional[list[str]]): Jobs we will exclude from running
+      jobs_allowlist (Optional[list[str]]): The only Jobs we will include to run
+    """
     super(TurbiniaCeleryWorker, self).__init__()
+    # Deregister jobs from denylist/allowlist.
+    job_manager.JobsManager.DeregisterJobs(jobs_denylist, jobs_allowlist)
+    disabled_jobs = list(config.DISABLED_JOBS) if config.DISABLED_JOBS else []
+    disabled_jobs = [j.lower() for j in disabled_jobs]
+    # Only actually disable jobs that have not been allowlisted.
+    if jobs_allowlist:
+      disabled_jobs = list(set(disabled_jobs) - set(jobs_allowlist))
+    if disabled_jobs:
+      log.info(
+          'Disabling non-allowlisted jobs configured to be disabled in the '
+          'config file: {0:s}'.format(', '.join(disabled_jobs)))
+      job_manager.JobsManager.DeregisterJobs(jobs_denylist=disabled_jobs)
+
+    # Check for valid dependencies/directories.
+    dependencies = config.ParseDependencies()
+    if config.DOCKER_ENABLED:
+      check_docker_dependencies(dependencies)
+    check_system_dependencies(dependencies)
     check_directory(config.MOUNT_DIR_PREFIX)
     check_directory(config.OUTPUT_DIR)
     check_directory(config.TMP_DIR)
+
+    jobs = job_manager.JobsManager.GetJobNames()
+    log.info(
+        'Dependency check complete. The following jobs will be enabled '
+        'for this worker: {0:s}'.format(','.join(jobs)))
     self.worker = self.task_manager.celery.app
 
   def start(self):
@@ -831,7 +1208,7 @@ class TurbiniaCeleryWorker(TurbiniaClient):
     self.worker.start(argv)
 
 
-class TurbiniaPsqWorker(object):
+class TurbiniaPsqWorker:
   """Turbinia PSQ Worker class.
 
   Attributes:
@@ -842,8 +1219,13 @@ class TurbiniaPsqWorker(object):
     TurbiniaException: When errors occur
   """
 
-  def __init__(self, *_, **__):
-    """Initialization for PSQ Worker."""
+  def __init__(self, jobs_denylist=None, jobs_allowlist=None):
+    """Initialization for PSQ Worker.
+
+    Args:
+      jobs_denylist (Optional[list[str]]): Jobs we will exclude from running
+      jobs_allowlist (Optional[list[str]]): The only Jobs we will include to run
+    """
     config.LoadConfig()
     psq_publisher = pubsub.PublisherClient()
     psq_subscriber = pubsub.SubscriberClient()
@@ -857,14 +1239,43 @@ class TurbiniaPsqWorker(object):
       log.error(msg)
       raise TurbiniaException(msg)
 
+    # Deregister jobs from denylist/allowlist.
+    job_manager.JobsManager.DeregisterJobs(jobs_denylist, jobs_allowlist)
+    disabled_jobs = list(config.DISABLED_JOBS) if config.DISABLED_JOBS else []
+    disabled_jobs = [j.lower() for j in disabled_jobs]
+    # Only actually disable jobs that have not been allowlisted.
+    if jobs_allowlist:
+      disabled_jobs = list(set(disabled_jobs) - set(jobs_allowlist))
+    if disabled_jobs:
+      log.info(
+          'Disabling non-allowlisted jobs configured to be disabled in the '
+          'config file: {0:s}'.format(', '.join(disabled_jobs)))
+      job_manager.JobsManager.DeregisterJobs(jobs_denylist=disabled_jobs)
+
+    # Check for valid dependencies/directories.
+    dependencies = config.ParseDependencies()
+    if config.DOCKER_ENABLED:
+      check_docker_dependencies(dependencies)
+    check_system_dependencies(dependencies)
     check_directory(config.MOUNT_DIR_PREFIX)
     check_directory(config.OUTPUT_DIR)
     check_directory(config.TMP_DIR)
 
+    jobs = job_manager.JobsManager.GetJobNames()
+    log.info(
+        'Dependency check complete. The following jobs are enabled '
+        'for this worker: {0:s}'.format(','.join(jobs)))
     log.info('Starting PSQ listener on queue {0:s}'.format(self.psq.name))
     self.worker = psq.Worker(queue=self.psq)
 
   def start(self):
     """Start Turbinia PSQ Worker."""
+    if config.PROMETHEUS_ENABLED:
+      if config.PROMETHEUS_PORT and config.PROMETHEUS_ADDR:
+        log.info('Starting Prometheus endpoint.')
+        start_http_server(
+            port=config.PROMETHEUS_PORT, addr=config.PROMETHEUS_ADDR)
+      else:
+        log.info('Prometheus enabled but port or address not set!')
     log.info('Running Turbinia PSQ Worker.')
     self.worker.listen()
